#!/bin/bash

# Скрипт lossless оптимизации изображений (без потери качества)
# Все пути - относительно скрипта
# Запуск: ./optimizm.sh

#
# Настройки
#

# База данных SQLite для хранения уже оптимизированных изображений
if [[ -z $OPTIMIZM_OPTIMIZM_DB_FILE ]]; then
  OPTIMIZM_DB_FILE="optimizm.db"
fi

# Пути к изображениям, разделенные пробелами
if [[ -z $OPTIMIZM_DIRS ]]; then
  OPTIMIZM_DIRS="www/upload www/images"
fi

# Режим вывода сообщений
if [[ -z $OPTIMIZM_VERBOSE ]]; then
  OPTIMIZM_VERBOSE=1
fi




###################################     Library     ###################################

# Сброс цвета
Color_Off='\033[0m'       # Text Reset

# Обычные цвета
Black='\033[0;30m'        # Black
Red='\033[0;31m'          # Red
Green='\033[0;32m'        # Green
Yellow='\033[0;33m'       # Yellow
Blue='\033[0;34m'         # Blue
Purple='\033[0;35m'       # Purple
Cyan='\033[0;36m'         # Cyan
White='\033[0;37m'        # White

# Жирные
BBlack='\e[1;30m'       # Black
BRed='\e[1;31m'         # Red
BGreen='\e[1;32m'       # Green
BYellow='\e[1;33m'      # Yellow
BBlue='\e[1;34m'        # Blue
BPurple='\e[1;35m'      # Purple
BCyan='\e[1;36m'        # Cyan
BWhite='\e[1;37m'       # White

# Вывод сообщений, если включен подробный режим
function echoVerbose {
  if [ "$OPTIMIZM_VERBOSE" -eq "1" ]; then
    echo -e $@
  fi
}

function echoNotice {
  echoVerbose "$BGreen""$@""$Color_Off"
}

function echoWarning {
  echoVerbose "$BYellow""$@""$Color_Off"
}

function echoInfo {
  echoVerbose "$BBlue""$@""$Color_Off"
}

function init {
  resp=`which sqlite3`

  if [[ $? != "0" ]]; then
    echo 'No sqlite3'
    exit 1
  fi

  resp=`which jpegoptim`

  if [[ $? != "0" ]]; then
    echo 'No jpegoptim. Run: sudo apt-get install jpegoptim gifsicle optipng'
    exit 1
  fi

  resp=`which gifsicle`

  if [[ $? != "0" ]]; then
    echo 'No gifsicle. Run: sudo apt-get install jpegoptim gifsicle optipng'
    exit 1
  fi

  resp=`which optipng`

  if [[ $? != "0" ]]; then
    echo 'No optipng. Run: sudo apt-get install jpegoptim gifsicle optipng'
    exit 1
  fi

  # Создает БД если ее еще нет
  sqlite3 $OPTIMIZM_DB_FILE "CREATE TABLE IF NOT EXISTS images (filepath VARCHAR(512) NOT NULL, md5_before CHAR(32) NOT NULL, md5_after CHAR(32) NOT NULL, size_before INTEGER NOT NULL, size_after INTEGER NOT NULL, run_id INTEGER NOT NULL, date DATETIME NOT NULL);"
}



function optimizeExt {

  local EXT=$1
  local CMD=$2

  echoNotice "Optimize *.$EXT"

  for DIR in $OPTIMIZM_DIRS; do

    if [[ ! -d "$DIR" ]]; then
      echoWarning "Directory not exists: $DIR"
      continue
    fi

    echoNotice "Entering directory $DIR"

    NUM_FILES=`find $DIR -type f -iname "*.$EXT" | wc -l`
    NUM_COUNTER=0


    find "$DIR" -type f -iname "*.$EXT" -print0 | while read -d $'\0' FILE; do
      let "NUM_COUNTER += 1"
      progressBar $NUM_FILES $NUM_COUNTER
      MD5_BEFORE=`md5sum "$FILE" | awk '{ print $1 }'`
      SIZE_BEFORE=`stat --printf="%s" "$FILE"`


      COUNT=`sqlite3 "$OPTIMIZM_DB_FILE" "SELECT COUNT(*) FROM images WHERE filepath = '${FILE//\'/''}' AND md5_after = '$MD5_BEFORE';"`
      if [[ "$COUNT" == "0" ]]; then
        # echo '+++'
        $CMD "$FILE" > /dev/null
        MD5_AFTER=`md5sum "$FILE" | awk '{ print $1 }'`
        SIZE_AFTER=`stat --printf="%s" "$FILE"`
        `sqlite3 "$OPTIMIZM_DB_FILE" "INSERT INTO images (filepath, md5_before, md5_after, size_before, size_after, run_id, date) VALUES ('${FILE//\'/''}', '$MD5_BEFORE', '$MD5_AFTER', $SIZE_BEFORE, $SIZE_AFTER, $RUN_ID, datetime());"`
      fi
    done
  done

}

function progressBar {
  if [ "$OPTIMIZM_VERBOSE" -ne "1" ]; then
    return
  fi
  local X=$1
  local Y=$2
  let "PERCENT = Y * 100 / X"
  echo -ne "$PERCENT%\r"
  if [[ $PERCENT == "100" ]]; then
    echo -ne '\n'
  fi
}

function printStatistics {
  query="SELECT ROUND((1 - 1.0 * SUM(size_after) / SUM(size_before)) * 100, 1) FROM images WHERE run_id = $RUN_ID;"
  ZIP_RATIO=`sqlite3 "$OPTIMIZM_DB_FILE" "$query"`
  if [[ -z "$ZIP_RATIO" ]]; then
    echoInfo "Изображения не были сжаты"
  else
    echoInfo "Изображения сжаты на $ZIP_RATIO%"
  fi
}

###################################     RUN     ###################################

cd "$(dirname "$0")"

init

# Номер запуска, используется для вычисления статистики запуска
RUN_ID=`date -u +%s`

optimizeExt 'jpg' 'jpegoptim --strip-all'
optimizeExt 'jpeg' 'jpegoptim --strip-all'
optimizeExt 'gif' 'gifsicle --careful -w -b -O2'
optimizeExt 'png' 'optipng -o5 -quiet -preserve'

printStatistics
